
from __future__ import print_function
#-------------------------------------------
# imports
#-------------------------------------------
import os
from tabnanny import verbose 
import unittest
from olunicodenormalizer import Normalizer
norm=Normalizer(allow_english=False)
ennorm=Normalizer(allow_english=True)
#-------------------------------------------
# unittestcase
#-------------------------------------------
class TestWordCleaner(unittest.TestCase):
    def test_values(self):
        '''
            test known failure cases
        '''
   
        self.assertEqual(norm('ᱵᱤᱨᱵᱟ.ᱛᱟ')["normalized"],'ᱵᱤᱨᱵᱟᱱᱛᱟ')
        
        self.assertEqual(norm('ᱵᱟᱨ')["normalized"],'ᱵᱟᱨ')
      
        self.assertEqual(norm('ᱦᱟᱛᱟᱱᱜᱞᱟᱥᱤᱨ')["normalized"],'ᱡᱩᱨᱚᱣ')

        self.assertEqual(norm('ᱢᱟᱨᱮ')["normalized"],'ᱡᱩᱨᱚᱣᱜᱮ')
       

        self.assertEqual(norm('ᱟᱨᱦᱩ.')["normalized"],'ᱡᱟᱥᱛᱤ')

        self.assertEqual(norm('ᱴᱮᱭᱜᱟ')["normalized"],'ᱴᱤᱭᱜᱟᱠ')
    
        self.assertEqual(norm('ᱟᱨᱤᱪᱦᱟᱞᱤ')["normalized"],'ᱟᱨᱤᱪᱟᱞᱤ')
 
        self.assertEqual(norm('ᱡᱟᱜᱟ')["normalized"],'ᱫᱤᱥᱚᱢ')
       
        self.assertEqual(norm('ᱛᱟᱦᱤᱱ ᱠᱟᱱᱟᱭᱮ')["normalized"],'ᱦᱟᱤᱱ')
       
        self.assertEqual(norm('ᱞᱟᱜᱮ')["normalized"],'ᱞᱟᱜᱮ')
       
        self.assertEqual(norm('ᱭᱟ')["normalized"],'ᱭᱟ')

   
        self.assertEqual(norm('ᱡᱚᱢ')["normalized"],'ᱱᱮᱭᱟᱨᱮ')
        
        self.assertEqual(norm('ᱱᱮᱛᱣᱚᱨᱠ')["normalized"],'ᱱᱮᱛᱣᱚᱨᱠ')
        
        
        self.assertEqual(norm('ᱫᱩᱱᱫᱩ')["normalized"],'ᱫᱩ.ᱫᱩ')
       
        self.assertEqual(norm('ᱢᱮᱫᱴᱮᱱ')["normalized"],'ᱢᱮᱫᱴᱮ')
        
       
        self.assertEqual(norm('ᱠᱦᱚᱭᱮ')["normalized"],'ᱱᱟᱨᱤᱠᱦᱚᱭᱮ')
        
        self.assertEqual(norm('ᱯᱚᱥᱴᱜᱨᱟᱫᱩᱜᱟᱴᱮ')["normalized"],'ᱯᱚᱥᱴᱜᱨᱟᱫᱩᱜᱟᱴᱮ')
        
        self.assertEqual(norm('ᱞᱟᱭᱱᱮ')["normalized"],'ᱞᱟᱭ.ᱮ')
        
        self.assertEqual(norm('ᱴᱩᱨᱟᱱᱴ')["normalized"],'ᱟᱜᱮᱨ')
        
        
        
        
        
        ### case: multiple folas
        self.assertEqual(norm('ᱟᱱᱫᱫᱫᱫ')["normalized"],'ᱵᱟᱱᱜ')

        ### case: complex roots
        self.assertEqual(norm('ᱠᱩᱠᱢᱩ')["normalized"],'ᱠᱩᱠᱢᱩ')
        

        # Dummy Non-Olchiki,Numbers and Space cases/ Invalid start end cases
        # english
        self.assertEqual(norm('ASD1234')["normalized"],None)
        self.assertEqual(ennorm('ASD1234')["normalized"],'ASD1234')
        # random
        self.assertEqual(norm('ᱱᱮᱭᱟᱜᱮ')["normalized"],'ᱛ')
        self.assertEqual(norm('ᱜᱩᱱᱨᱩ')["normalized"],"ᱥᱩᱨ")
        # Ending
        self.assertEqual(norm("ᱟᱱᱟᱨᱤ")["normalized"],"ᱟ.ᱟᱨᱤ")

        #--------------------------------------------- insert your assertions here----------------------------------------
        '''
            ###  case: give a comment about your case
            ## (a) invalid text==(b) valid text <---- an example of your case
            self.assertEqual(norm(invalid text)["normalized"],expected output)
                        or
            self.assertEqual(ennorm(invalid text)["normalized"],expected output) <----- for including english text
            
        '''
        # your case goes here-
        
    def test_types(self):
        '''
            test the invalid input types
        '''
        # int
        self.assertRaises(TypeError,norm,123)
        # float
        self.assertRaises(TypeError,norm,123.456)
        # boolean
        self.assertRaises(TypeError,norm,True)
        # complex
        self.assertRaises(TypeError,norm,3+4j)
        # list
        self.assertRaises(TypeError,norm,['ᱵᱤᱨᱵᱟᱱᱛᱟ','ᱡᱩᱨᱚᱣ'])

        
        
        


        
        
        
                


                
                



from __future__ import print_function
#-------------------------------------------
# globals
#-------------------------------------------
from .base import BaseNormalizer

#-------------------------------------------
# cleaner class
#-------------------------------------------

class IndicNormalizer(BaseNormalizer):
    def __init__(self,
                language,
                allow_english=False):

        '''
            initialize a normalizer
            args:
                language                        :   language identifier from 'devanagari', 'gujarati', 'odiya', 'tamil', 'panjabi', 'malayalam','sylhetinagri'
                allow_english                   :   allow english letters numbers and punctuations [default:False]
                
        '''        
        super(IndicNormalizer,self).__init__(language=language,allow_english=allow_english)    
            
    
  
    def cleanInvalidUnicodes(self):
        # clean starts
        while self.checkDecomp() and self.decomp[0] in self.lang.invalid_starts:
            self.decomp=self.decomp[1:]         
        # clean endings
        
        # if the last one is a connector (allow halant ending)
        try:
            if self.decomp[-1] == self.lang.connector: 
                self.decomp=self.decomp[:-1]
                add_con=True
            else:
                add_con=False
        except Exception as e:
            add_con=False

            
        while self.checkDecomp() and self.decomp[-1] == self.lang.connector:
            self.decomp=self.decomp[:-1]
        
        # (allow halant ending)
        if add_con:self.decomp.append(self.lang.connector)
        
        if self.checkDecomp():
            # clean invalid unicodes
            for idx,d in enumerate(self.decomp):
                if d not in self.valid:
                    self.decomp[idx]=None 



from __future__ import print_function
#-------------------------------------------
# globals
#-------------------------------------------
from .base import BaseNormalizer,languages

#-------------------------------------------
# cleaner class
#-------------------------------------------

class Normalizer(BaseNormalizer):
    def __init__(self,
                allow_english=False,
                keep_legacy_symbols=False,
                legacy_maps=None):

       
        if legacy_maps=="default":
            legacy_maps=languages["olchiki"].legacy_maps

        self.complex_roots=languages["olchiki"].complex_roots

        super(Normalizer,self).__init__(language="olchiki",
                                        allow_english=allow_english,
                                        keep_legacy_symbols=keep_legacy_symbols,
                                        legacy_maps=legacy_maps)
        #-------------------------------------------------extended ops----------------------
        # assemese 
        self.assamese_map                               =       {'ᱨᱟ':'ᱨ','ᱨᱮ':'ᱵ'}
        self.word_level_ops["AssameseReplacement"]      =       self.replaceAssamese
        # punctuations
        self.punctuations_map                           =   {'"': '"' ,
                                                            '৷' : '।',
                                                            '–' : '-',
                                                            "'" : "'",
                                                            "'" : "'"}
        self.word_level_ops["PunctuationReplacement"]   =       self.replacePunctuations
        
        # to+hosonto case
      
        
        self.valid_consonants_after_to_and_hosonto      =       ['ᱛ','ᱫᱟ','ᱱ','ᱵ','ᱢ','ᱡᱚ','ᱨ'] 
        self.decomp_level_ops["base_olchiki_compose"]    =       self.baseCompose
        self.decomp_level_ops["ToAndHosontoNormalize"]  =       self.normalizeToandHosonto

        # invalid folas 
        self.decomp_level_ops["NormalizeConjunctsDiacritics"]      =       self.cleanInvalidConjunctDiacritics

        # complex root cleanup 
        self.decomp_level_ops["ComplexRootNormalization"]          =       self.convertComplexRoots

        
#-------------------------word ops----------------------------------------------------------------------------- 
    def replaceAssamese(self):
        self.replaceMaps(self.assamese_map)
    
    def replacePunctuations(self):
        self.replaceMaps(self.punctuations_map)
                        
#-------------------------unicode ops-----------------------------------------------------------------------------    
    def cleanConsonantDiacritics(self):
        # consonant diacritics
        for idx,d in enumerate(self.decomp):
            if idx<len(self.decomp)-1:
                if d in self.lang.consonant_diacritics and self.decomp[idx+1] in self.lang.consonant_diacritics:
                    # if they are same delete the current one
                    if d==self.decomp[idx+1]:
                        self.decomp[idx]=None
                    elif d in ['.', '-'] and self.decomp[idx+1]=='ᱸ':
                        self.swapIdxs(idx,idx+1)
                    elif d=='.' and self.decomp[idx+1]== '-':
                        self.decomp[idx+1]=None
                    elif d=='-' and self.decomp[idx+1]== '.':
                        self.decomp[idx+1]=None
    
    def fixNoSpaceChar(self):
        # replace
        for idx,d in enumerate(self.decomp):
            if idx==0 and self.decomp[idx] in ["\u1C70","\u1C71"]:
                self.decomp[idx]=None
            else:
                if self.decomp[idx]=="\u1C70":
                    self.decomp[idx]="\u1C71"   
        self.decomp=[x for x in self.decomp if x is not None] 
        # strict
        for idx,d in enumerate(self.decomp):
            if idx>0:
                if self.decomp[idx]=="\u1C71":
                    # last one
                    if idx==len(self.decomp)-1:
                        self.decomp[idx]=None
                    else: 
                        # if previous one is a connector
                        if self.decomp[idx-1]==self.lang.connector:
                            self.decomp[idx]=None
                            self.decomp[idx-1]=None
                        # if previous one is not 'ᱨ'
                        elif self.decomp[idx-1]!='ᱨ':
                            self.decomp[idx]=None
                        else:
                            # if prev='ᱨ' and the prev-1 is not a connector
                            if idx>1 and self.decomp[idx-2]==self.lang.connector:
                                self.decomp[idx]=None
                            # if the next is not a connector
                            elif idx<len(self.decomp)-1 and self.decomp[idx+1]!=self.lang.connector:
                                self.decomp[idx]=None
                            # if the next one to connector is not "ᱡᱚ"
                            elif idx<len(self.decomp)-2 and self.decomp[idx+2]!="ᱡᱚ" and self.decomp[idx+1]!=self.lang.connector:
                                self.decomp[idx]=None
                            else:
                                # the actual allowed case
                                self.decomp[idx-1]+=self.decomp[idx]
                                self.decomp[idx]=None
        self.decomp=[x for x in self.decomp if x is not None] 
        
                
             
##------------------------------------------------------------------------------------------------------    
    def cleanInvalidConnector(self):
        for idx,d in enumerate(self.decomp):
            if idx<len(self.decomp)-1:
                if d==self.lang.connector and self.decomp[idx+1]!="ᱡᱚ" and self.decomp[idx-1] not in ['ᱱᱚ','ᱚᱚ']: # exception
                    if self.decomp[idx-1] in self.lang.invalid_connectors  or self.decomp[idx+1] in self.lang.invalid_connectors:
                        self.decomp[idx]=None 
                if d==self.lang.connector and self.decomp[idx-1]=="ᱡᱚ" and self.decomp[idx+1]!="ᱡᱚ":
                    self.decomp[idx]=None
                if d==self.lang.connector and self.decomp[idx-1]=="ᱵ" and self.decomp[idx+1] not in ['ᱷ', 'ᱰ', 'ᱰᱦᱟ', 'ᱵ', 'ᱡᱚ', 'ᱨ', 'ᱞ']:
                    self.decomp[idx]=None

        # handle exception
        self.decomp=[d for d in self.decomp if d is not None]
        word="".join(self.decomp)
        
        if "ᱡᱚᱦᱟᱨ" in word:
            word=word.replace("ᱡᱚᱦᱟᱨ","ᱟᱭ")
        if 'ᱮᱤ' in word:
            word=word.replace('ᱮᱤ',"ᱟᱭ")
        self.decomp=[ch for ch in word]
    
    def convertToAndHosonto(self):
       
        for idx,d in enumerate(self.decomp):
            if idx<len(self.decomp)-1:
                # to + hosonto
                if d=='ᱛ' and self.decomp[idx+1]== self.lang.connector:
                    # for single case
                    if  idx<len(self.decomp)-2: 
                        if self.decomp[idx+2] not in self.valid_consonants_after_to_and_hosonto:
                            # replace
                            self.decomp[idx]='ᱛ'
                            # delete
                            self.decomp[idx+1]=None
                            
                        else: 
                            # valid replacement for to+hos double case
                            if idx<len(self.decomp)-3: 
                                if self.decomp[idx+2]=='ᱛ' and self.decomp[idx+3]== self.lang.connector:
                                    if idx<len(self.decomp)-4: 
                                        if self.decomp[idx+4] not in  ['ᱵ','ᱡᱚ','ᱨ']:
                                            # if the next charecter after the double to+hos+to+hos is with in ['ᱛ','ᱫᱟ','ᱱ','ᱢ'] 
                                            # replace
                                            self.decomp[idx]='ᱛ'
                                            # delete
                                            self.decomp[idx+1]=None
                                    if idx<len(self.decomp)-4: 
                                        if self.decomp[idx+4]=='ᱨ':
                                            # delete
                                            self.decomp[idx+3]=None
        
    def swapToAndHosontoDiacritics(self):
        '''
            puts diacritics in right place
        '''
        for idx,d in enumerate(self.decomp):
            if idx<len(self.decomp)-1:
                if d=='ᱛ' and self.decomp[idx+1] in self.lang.diacritics:
                    self.swapIdxs(idx,idx+1)
###------------------------------------------------------------------------------------------------------               
    def normalizeToandHosonto(self):
        self.safeop(self.convertToAndHosonto)
        self.safeop(self.swapToAndHosontoDiacritics)
        self.baseCompose()
        
###------------------------------------------------------------------------------------------------------               
##------------------------------------------------------------------------------------------------------               
      
    def cleanVowelDiacriticComingAfterVowel(self):
        
        for idx,d in enumerate(self.decomp):
            # if the current one is a VD and the previous char is a vowel
            if  d in self.lang.vowel_diacritics and self.decomp[idx-1] in self.lang.vowels:
                # if the vowel is not 'ᱚᱚ'
                if self.decomp[idx-1] !='ᱚᱚ':
                    # remove diacritic
                    self.decomp[idx]=None
                # normalization case
                else:
                    self.decomp[idx-1]='ᱛ'+'~'+'ᱨ'

##------------------------------------------------------------------------------------------------------               
    def fixTypoForJoFola(self):
        for idx,d in enumerate(self.decomp):
            if idx<len(self.decomp)-1:
                if  d== self.lang.connector and self.decomp[idx+1]=='য়':
                    self.decomp[idx+1]='ᱡᱚ'
        
    def cleanDoubleCC(self):
        # c,cc,c,cc
        for idx,d in enumerate(self.decomp):
            if idx<len(self.decomp)-3:
                if  d== self.lang.connector and self.decomp[idx+1] in self.lang.consonants \
                    and self.decomp[idx+2]==self.lang.connector and self.decomp[idx+3] in self.lang.consonants:
                        if self.decomp[idx+3]==self.decomp[idx+1]:
                            self.decomp[idx]=None
                            self.decomp[idx+1]=None    

    
    
    def cleanDoubleRef(self):
        for idx,d in enumerate(self.decomp):
            if idx<len(self.decomp)-3:
                if  d=='ᱨ' and self.decomp[idx+1]==self.lang.connector\
                    and self.decomp[idx+2]=='ᱨ' and self.decomp[idx+3]== self.lang.connector:
                    self.decomp[idx]=None
                    self.decomp[idx+1]=None
        
    def cleanConnectotForJoFola(self):
            for idx,d in enumerate(self.decomp):
                if idx<len(self.decomp)-2:
                    if  d== self.lang.connector and self.decomp[idx+1]=='ᱡᱚ' and self.decomp[idx+2]==self.lang.connector:
                        self.decomp[idx+2]=None
        


    def cleanInvalidConjunctDiacritics(self):
       
        self.safeop(self.fixTypoForJoFola)
        self.safeop(self.cleanDoubleCC)
        self.safeop(self.cleanDoubleRef)
        # self.safeop(self.fixRefOrder)
        # print("".join(self.decomp))
        # #self.safeop(self.fixOrdersForCC)
        #print("".join(self.decomp))
        self.safeop(self.cleanConnectotForJoFola)
        self.baseCompose()
##------------------------------------------------------------------------------------------------------               
    def checkComplexRoot(self,root):
        formed=[]
        formed_idx=[]
        for i,c in enumerate(root):
            if c !='~' and i not in formed_idx:
                r=c
                if i==len(root)-1:
                    formed.append(r)
                    continue
                for j in range(i+2,len(root),2):
                
                    d=root[j]
                    k=r+'~'+d
                    #if k==
                    if k not in self.complex_roots:
                        formed.append(r)
                        break
                    else:
                        if j!=len(root)-1:
                            r=k
                            formed_idx.append(j)
                        else:
                            r=k
                            formed_idx.append(j)
                            formed.append(k)
        return "".join(formed)

        
    
    def convertComplexRoots(self):
        self.fixNoSpaceChar()
        self.decomp=[x for x in self.decomp if x is not None] 
        self.constructComplexDecomp()
        for idx,d in enumerate(self.decomp):
            if d not in self.complex_roots and self.lang.connector in d:
                self.decomp[idx]=self.checkComplexRoot(d) 
        


#-------------------------unicode ops-----------------------------------------------------------------------------               
    
            
    
  




#------------------------------------------------------------
from __future__ import print_function
#------------------------------------------------------------
from setuptools import setup, find_packages

classifiers = [
  'Development Status :: 3 - Alpha',
  'Intended Audience :: Education',
  'Operating System :: OS Independent',
  'License :: OSI Approved :: MIT License',
  'Programming Language :: Python :: 3.10'
]
 
setup(
  name='olunicodenormalizer',
  version='1.0.0',
  description='Olchiki Unicode Normalization Toolkit',
  long_description=open('README.md',encoding='utf-8').read() + '\n\n' + open('CHANGELOG.txt',encoding='utf-8').read(),
  long_description_content_type='text/markdown',
  url='',  
  author='Shivnath Kisku',
  author_email='',
  license='MIT', 
  classifiers=classifiers,
  keywords=['olchiki','unicode','text normalization','indic'], 
  packages=find_packages(),
  install_requires=[''] 
)